#!/usr/bin/env bash

set -e

unset LD_{RUN,LIBRARY}_PATH LIBRARY_PATH CPATH {C,CPLUS}_INCLUDE_PATH PYTHONPATH
unset EUPS_DIR EUPS_SHELL EUPS_PATH EUPS_PKGROOT SETUP_EUPS

source "$(dirname "$0")/lib/install.rc"

source "$(dirname "$0")/versions"
source "$(dirname "$0")/variables"

if [[ nproc -ge 2 ]]
then
  export MAKEFLAGS="-j $((nproc))"
  export EUPSPKG_SCONSFLAGS="-j $((nproc))"
fi

workdir="${workdir:-/tmp}"/hscpipe_build_dir

eups_path="${prefix}/stack_${hscpipe_ver}"


main()
(
  export PATH="$path"
  install_gcc
  export PATH="$prefix/bin:$PATH"

  install_binutils
  install_git
  install_sqlite3
  install_tcltk
  install_python

  install_openblas
  install_numpy

  # In addition to numpy, we need to pre-install several python libraries.
  # To know what to pre-install:
  #     'find {stack_dir}/Linux64 -name check_python_module'
  pip install scipy
  pip install matplotlib
  pip install astropy

  export EUPS_PKGROOT="${eups_pkgroot}"

  mkdir -p "${eups_path}"
  pushd "${eups_path}"

  install_eups

  # We replace the build directory (EupsBuildDir)
  # with a symbolic link to a subdirectory of $workdir
  if :
  then
    rm -rf EupsBuildDir
    mkdir -p "$workdir/EupsBuildDir/Linux64"
    mkdir EupsBuildDir
    ln -s "$workdir/EupsBuildDir/Linux64" EupsBuildDir/
  fi

  popd

  cleaner_thread &

  source "${eups_path}"/loadLSST.bash
  (
    eups distrib install hscPipe "${hscpipe_ver}"
  )

  (
    eups distrib install -j xpa 2.1.15.lsst3

    # set up required packages (see display_ds9.table)
    # before installing display_ds9 with '-j' option.
    # With '-j' option, packages set up currently will be
    # inscribed in ups_db/Linux64/display_ds9/version/display_ds9.table
    setup afw
    setup xpa
    setup python_future
    eups distrib install -j display_ds9 13.0+5
  )

  rm -rf "${eups_path}"/EupsBuildDir

  strip_hack_from_gcc

  # install bashrc
  cat "$(dirname "$0")"/rsrc/bashrc.in |
    sed s$'\xff'@prefix@$'\xff'"$prefix"$'\xff'g |
    sed s$'\xff'@eups_path@$'\xff'"$EUPS_PATH"$'\xff'g |
    sed s$'\xff'@hscpipe_ver@$'\xff'"$hscpipe_ver"$'\xff'g \
    > "$prefix"/bashrc

  # setup-eups() in bashrc needs $prefix/bin/check-requisites-hscPipe-${hscpipe_ver},
  # which will be created by post_build.sh. For users who won't run
  # post_build.sh, we place an empty exe there

  echo "int main(){ return 0; }" > "$workdir"/check-requisites.c
  gcc -O3 "$workdir"/check-requisites.c \
    -o "${prefix}/bin/check-requisites-hscPipe-${hscpipe_ver}"
)

cleaner_thread()
(
  cd "${eups_path}"
  while [[ -e EupsBuildDir ]]
  do
    sleep 60
    dirs=($(ls -d -t -1 EupsBuildDir/Linux64/*))
    # remove all the dirs except the newest ones
    for((i = 3; i < ${#dirs[@]}; ++i))
    do
      rm -rf "${dirs[i]}"
    done
  done
)

main
